@@ -18,10 +18,14 @@
 #undef	LIBGLE_VERSION
 #undef	VERSION
 
+#undef	GLE_WORKS_WITH_GTK_VERSION
+
 @BOTTOM@
 /* general defines
  */
 #define	GLE_FLASH_MSEC		(333)
 #define	GLE_PRIVATE_KEY(key)	"__gle-private-data__-" #key
 #define	GLE_GARGS_PREALLOC	(512)
+#define	GLE_GOBJECTS_PREALLOC	(16)
+#define	GLE_GWIDGETS_PREALLOC	(64)
 #define	DMALLOG
