@@ -30,13 +30,21 @@ extern "C" {
 
 
 
-
+/* --- global variables --- */
 extern	const guint	gle_libversion_major;
 extern	const guint	gle_libversion_revision;
 extern	const guint	gle_libversion_age;
 extern	const gchar	*gle_libversion;
 
 
+/* --- GLE main part --- */
+void            gle_init                        (int            *argc,
+						 char           ***argv);
+const gchar*    gle_prg_name                    (void);
+void            gle_finish                      (void);
+
+
+
 
 
 
