@@ -29,14 +29,20 @@ extern "C" {
 
 
 
-
-GtkWidget*	gle_menu_widegt_ops_popup	(GtkWidget	*widget,
+/* --- prototypes --- */
+GtkWidget*	gle_menu_widget_ops_new		(void);
+void		gle_menu_widget_ops_popup	(GtkMenu	*menu,
+						 GtkWidget	*widget,
 						 guint		 x,
 						 guint		 y,
+						 guint		 mouse_button,
 						 guint32	 time);
 
 
 
+
+
+
 #ifdef __cplusplus
 #pragma {
 }
