@@ -18,14 +18,15 @@
 #include	"config.h"
 
 #include	"gleshell.h"
-#include	"glepublic.h"
 #include	"gleprivate.h"
 #include	"glegobject.h"
 #include	"glegwidget.h"
 #include	"gleselector.h"
 #include	"glemenus.h"
 #include	"gleeditor.h"
+#include	"gle.h"
 #include	<gdk/gdk.h>
+#include	<string.h>
 
 
 
@@ -42,7 +43,7 @@ enum
   CLIST_COL_TYPE,
   CLIST_COL_STATE,
   CLIST_COL_REFCOUNT,
-  CLIST_COL_GLENAME,
+  CLIST_COL_GNAME,
   CLIST_COL_NAME,
   CLIST_N_COLS
 };
@@ -51,7 +52,7 @@ static guint	clist_init_col_width[CLIST_N_COLS] =
   50	/* CLIST_COL_TYPE */,
   50	/* CLIST_COL_STATE */,
   50	/* CLIST_COL_REFCOUNT */,
-  50	/* CLIST_COL_GLENAME */,
+  50	/* CLIST_COL_GNAME */,
   50	/* CLIST_COL_NAME */,
 };
 #define	CLIST_INITIAL_COL_WIDTH(x)	(clist_init_col_width[(x)])
@@ -65,12 +66,12 @@ struct _GleRowData
   GleShell *shell;
   GtkWidget *widget;
   gint row;
+  gint gle_name_handler;
   gint destroy_handler;
 };
 
 
 /* --- prototypes --- */
-static guint		gle_shell_get_type		(void);
 static void		gle_shell_class_init		(GleShellClass	*class);
 static void		gle_shell_init			(GleShell	*shell);
 static void		gle_shell_set_customer_name	(GleShell	*shell);
@@ -85,7 +86,15 @@ static void		gle_shell_row_selected		(GtkCList	*clist,
 static void		gle_shell_customer_destroyed	(GtkWidget	*customer,
 							 GleShell	*shell);
 static void		gle_shell_clist_append_widget	(GtkWidget	*widget,
-							 GleShell   *shell);
+							 GleShell	*shell);
+static void		gle_row_data_chunk_free		(GtkCList       *clist,
+							 guint           row,
+							 gpointer       *row_data_p,
+							 gpointer        func_data);
+static void		gle_row_data_update		(GtkCList       *clist,
+							 guint           row,
+							 gpointer       *row_data_p,
+							 gpointer        func_data);
 
 
 /* --- variables --- */
@@ -94,10 +103,10 @@ static GleShell		*gle_shell = NULL;
 
 
 /* --- functions --- */
-static guint
+GtkType
 gle_shell_get_type (void)
 {
-  static guint shell_type = 0;
+  static GtkType shell_type = 0;
   
   if (!shell_type)
     {
@@ -184,14 +193,14 @@ gle_shell_init (GleShell *shell)
     "Type"	/* CLIST_COL_TYPE */,
     "State"	/* CLIST_COL_STATE */,
     "RefCount"	/* CLIST_COL_REFCOUNT */,
-    "GleName"	/* CLIST_COL_GLENAME */,
+    "GleName"	/* CLIST_COL_GNAME */,
     "Name"	/* CLIST_COL_NAME */,
   };
 
   shell->customer = NULL;
   shell->customer_destroy_handler = 0;
 
-  string = g_strconcat ("GLE Engine", "	 [", gle_prg_name (), "]", NULL);
+  string = g_strconcat ("GLE-Shell", "	 [", gle_prg_name (), "]", NULL);
   gtk_widget_set (GTK_WIDGET (shell),
 		  "GtkWidget::name", "GLE-Shell",
 		  "GtkWindow::title", string,
@@ -300,7 +309,6 @@ gle_shell_init (GleShell *shell)
     gtk_widget_new (gle_selector_get_type (),
 		    "GtkObject::object_signal::candidate_check", gle_shell_candidate_check, shell,
 		    "GtkObject::signal::destroy", gtk_widget_destroyed, &shell->selector,
-		    "GtkWindow::title", "GLE - Selector",
 		    NULL);
 }
 
@@ -326,8 +334,7 @@ gle_shell_destroy (GtkObject *object)
   
   gle_root_shutdown ();
   
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  GTK_OBJECT_CLASS (parent_class)->destroy (object);
 
   gle_shell = NULL;
 }
@@ -355,43 +362,63 @@ gle_shell_get (void)
       gle_shell = GLE_SHELL (gtk_widget_new (gle_shell_get_type (), NULL));
       GLE_TAG (gle_shell);
       GLE_TAG (gle_shell->list_popup);
-      /* FIXME: GLE_TAG (gle_shell->selector); */
+      GLE_TAG (gle_shell->selector);
+      GLE_WEAK_TAG (GLE_SELECTOR (gle_shell->selector)->warning_window);
     }
 
   return gle_shell;
 }
 
-void
-gle_shell_popup (GleShell *shell)
-{
-  g_return_if_fail (shell != NULL);
-  g_return_if_fail (GLE_IS_SHELL (shell));
-  
-  if (!GTK_WIDGET_VISIBLE (shell))
-    gtk_widget_show (GTK_WIDGET (shell));
-
-  gdk_window_raise (GTK_WIDGET (shell)->window);
-}
-
 void
 gle_shell_popup_selector (GleShell	 *shell)
 {
+  gboolean do_selection;
   GtkWidget *customer;
 
   g_return_if_fail (shell != NULL);
   g_return_if_fail (GLE_IS_SHELL (shell));
 
-  gle_shell_set_customer (shell, NULL);
+  do_selection = !GLE_SELECTOR (shell->selector)->in_selection;
 
-  customer = gle_selector_make_selection (GLE_SELECTOR (shell->selector));
-
-  if (customer && GTK_OBJECT (customer)->ref_count > 1)
+  if (do_selection && gdk_pointer_is_grabbed () &&
+      !GLE_SELECTOR (shell->selector)->warning_visible)
     {
-      gle_selector_reset (GLE_SELECTOR (shell->selector));
-      gle_shell_set_customer (shell, customer);
+      gle_selector_popup_warning (GLE_SELECTOR (shell->selector));
+      do_selection = FALSE;
+    }
+
+  if (do_selection)
+    {
+      gle_shell_set_customer (shell, NULL);
+      
+      if (!gdk_pointer_is_grabbed () &&
+	  !GTK_WIDGET_VISIBLE (shell))
+	{
+	  gtk_widget_show (GTK_WIDGET (shell));
+	  gdk_window_raise (GTK_WIDGET (shell)->window);
+	}
+      
+      gtk_object_ref (GTK_OBJECT (shell));
+      
+      customer = gle_selector_make_selection (GLE_SELECTOR (shell->selector));
+      
+      if (!GTK_OBJECT_DESTROYED (shell) &&
+	  shell->selector)
+	{
+	  if (customer && GTK_OBJECT (customer)->ref_count > 1)
+	    {
+	      gle_selector_reset (GLE_SELECTOR (shell->selector));
+	      gle_shell_set_customer (shell, customer);
+	    }
+	  else
+	    gle_selector_reset (GLE_SELECTOR (shell->selector));
+	  
+	  if (!GTK_WIDGET_VISIBLE (shell))
+	    gtk_widget_show (GTK_WIDGET (shell));
+	}
+
+      gtk_object_unref (GTK_OBJECT (shell));
     }
-  else
-    gle_selector_reset (GLE_SELECTOR (shell->selector));
 }
 
 static void
@@ -502,12 +529,15 @@ gle_row_data_chunk_free (GtkCList	*clist,
       mem_chunk = (GMemChunk*) func_data;
       row_data = *row_data_p;
 
-      /* printf ("clear: row_data(%p) widget(%p) row(%d) hid(%d)\n", row_data, row_data->widget, row_data->row, row_data->destroy_handler); */
+      /* printf ("clear: row_data(%p) widget(%p) row(%d) gle-hid(%d) d-hid(%d)\n", row_data, row_data->widget, row_data->row, row_data->gle_name_handler, row_data->destroy_handler); */
 
       g_return_if_fail (row_data->row >= 0);
 
       *row_data_p = NULL;
-      gtk_signal_disconnect (GTK_OBJECT (row_data->widget), row_data->destroy_handler);
+      if (row_data->gle_name_handler)
+	gtk_signal_disconnect (GTK_OBJECT (row_data->widget), row_data->gle_name_handler);
+      if (row_data->destroy_handler)
+	gtk_signal_disconnect (GTK_OBJECT (row_data->widget), row_data->destroy_handler);
       g_mem_chunk_free (mem_chunk, row_data);
     }
 }
@@ -521,7 +551,7 @@ gle_shell_update_tree (GleShell	      *shell)
   g_return_if_fail (GLE_IS_SHELL (shell));
 
   gtk_clist_freeze (GTK_CLIST (shell->clist));
-  gtk_clist_row_foreach (GTK_CLIST (shell->clist),
+  gtk_clist_foreach_row (GTK_CLIST (shell->clist),
 			 gle_row_data_chunk_free,
 			 shell->row_data_mem_chunk);
   gtk_clist_clear (GTK_CLIST (shell->clist));
@@ -534,6 +564,9 @@ gle_shell_update_tree (GleShell	      *shell)
       g_free (shell->clist_indent);
       shell->clist_indent = NULL;
     }
+  gtk_clist_foreach_row (GTK_CLIST (shell->clist),
+			 gle_row_data_update,
+			 NULL);
 
   gtk_clist_thaw (GTK_CLIST (shell->clist));
 }
@@ -554,77 +587,150 @@ gle_row_data_destroyed (GtkWidget *widget,
   row_data->row = -44;
   g_return_if_fail (row >= 0);
   gtk_clist_set_row_data (clist, row, NULL);
-  gtk_signal_disconnect (GTK_OBJECT (row_data->widget), row_data->destroy_handler);
-  gtk_clist_set_text (clist, row, CLIST_COL_REFCOUNT, "????");
+  if (row_data->gle_name_handler)
+    gtk_signal_disconnect (GTK_OBJECT (row_data->widget), row_data->gle_name_handler);
+  if (row_data->destroy_handler)
+    gtk_signal_disconnect (GTK_OBJECT (row_data->widget), row_data->destroy_handler);
+  gtk_clist_set_text (clist, row, CLIST_COL_REFCOUNT, "?????");
   gtk_clist_set_text (clist, row, CLIST_COL_STATE, "Destroyed");
   gtk_clist_set_foreground (clist, row, & GTK_WIDGET (clist)->style->fg[GTK_STATE_INSENSITIVE]);
   gtk_clist_set_background (clist, row, & GTK_WIDGET (clist)->style->bg[GTK_STATE_INSENSITIVE]);
   g_mem_chunk_free (shell->row_data_mem_chunk, row_data);
 }
 
+static void
+gle_row_data_gle_name_changed (GtkWidget *widget,
+			       GleRowData *row_data)
+{
+  gle_row_data_update (GTK_CLIST (row_data->shell->clist),
+		       row_data->row,
+		       (gpointer*) &row_data,
+		       NULL);
+}
+
+static void
+gle_row_data_update (GtkCList       *clist,
+		     guint           row,
+		     gpointer       *row_data_p,
+		     gpointer        func_data)
+{
+  GleRowData *row_data;
+
+  g_return_if_fail (clist != NULL);
+  g_return_if_fail (GTK_IS_CLIST (clist));
+
+  row_data = *row_data_p;
+  
+  if (row_data)
+    {
+      GtkWidget *widget;
+      gchar *text[CLIST_N_COLS] = { NULL };
+      const gchar *s;
+      guint i;
+
+      g_return_if_fail (row_data->row == row);
+
+      widget = row_data->widget;
+      if (GTK_OBJECT_DESTROYED (widget))
+	{
+	  gle_row_data_destroyed (widget, row_data);
+	  return;
+	}
+
+      text[CLIST_COL_TYPE] = NULL;
+      gtk_clist_get_text (clist, row, CLIST_COL_TYPE, &text[CLIST_COL_TYPE]);
+      s = strchr (text[CLIST_COL_TYPE], ' ');
+      text[CLIST_COL_TYPE] = g_strconcat (g_filled_string (NULL, ' ', s ? s - text[CLIST_COL_TYPE] : 0),
+					  gtk_type_name (GTK_OBJECT_TYPE (widget)),
+					  NULL);
+
+      text[CLIST_COL_REFCOUNT] = g_new (gchar, 32);
+      sprintf (text[CLIST_COL_REFCOUNT], "%05d", GTK_OBJECT (widget)->ref_count);
+
+      s = NULL /* FIXME */;
+      text[CLIST_COL_GNAME] = g_strconcat ("\"",
+					   s ? s : "",
+					   "\"",
+					   NULL);
+
+      text[CLIST_COL_NAME] = g_strconcat ("\"",
+					  widget->name ? widget->name : "",
+					  "\"",
+					  NULL);
+
+      switch (widget->state)
+	{
+	case  GTK_STATE_ACTIVE:		text[CLIST_COL_STATE] = g_strdup ("(x) Active"); break;
+	case  GTK_STATE_PRELIGHT:		text[CLIST_COL_STATE] = g_strdup ("(x) Prelight"); break;
+	case  GTK_STATE_SELECTED:		text[CLIST_COL_STATE] = g_strdup ("(x) Selected"); break;
+	case  GTK_STATE_INSENSITIVE:	text[CLIST_COL_STATE] = g_strdup ("(x) Insensitive"); break;
+	default:				text[CLIST_COL_STATE] = g_strdup ("(x) Normal"); break;
+	}
+      text[CLIST_COL_STATE][1] = GTK_WIDGET_VISIBLE (widget) ? 'V' : 'H';
+
+      for (i = 0; i < CLIST_N_COLS; i++)
+	{
+	  gint new_width;
+	  
+	  new_width = gdk_text_width (GTK_WIDGET (clist)->style->font, text[i], strlen (text[i]));
+	  if (clist->column[i].width < new_width)
+	    gtk_clist_set_column_width (clist, i, new_width);
+	  gtk_clist_set_text (clist, row_data->row, i, text[i]);
+
+	  g_free (text[i]);
+	}
+    }
+}
+
 static void
 gle_shell_clist_append_widget (GtkWidget  *widget,
 			       GleShell	  *shell)
 {
   GtkCList *clist;
   GleRowData *row_data;
-  gchar *text[CLIST_N_COLS];
-  const gchar *s;
+  gchar *text[CLIST_N_COLS] = { NULL };
   guint i;
-
+  
   clist = GTK_CLIST (shell->clist);
-
+  
+  for (i = 0; i < CLIST_N_COLS; i++)
+    text[i] = NULL;
+  
   text[CLIST_COL_TYPE] = g_strconcat (shell->clist_indent,
 				      gtk_type_name (GTK_OBJECT_TYPE (widget)),
 				      NULL);
-  text[CLIST_COL_REFCOUNT] = g_new (gchar, 32);
-  sprintf (text[CLIST_COL_REFCOUNT], "%04d", GTK_OBJECT (widget)->ref_count);
-  s = gle_widget_get_glename (widget);
-  text[CLIST_COL_GLENAME] = g_strconcat ("\"",
-					 s ? s : "",
-					 "\"",
-					 NULL);
-  text[CLIST_COL_NAME] = g_strconcat ("\"",
-				      widget->name ? widget->name : "",
-				      "\"",
-				      NULL);
-  switch (widget->state)
-    {
-    case  GTK_STATE_ACTIVE:		text[CLIST_COL_STATE] = g_strdup ("(x) Active"); break;
-    case  GTK_STATE_PRELIGHT:		text[CLIST_COL_STATE] = g_strdup ("(x) Prelight"); break;
-    case  GTK_STATE_SELECTED:		text[CLIST_COL_STATE] = g_strdup ("(x) Selected"); break;
-    case  GTK_STATE_INSENSITIVE:	text[CLIST_COL_STATE] = g_strdup ("(x) Insensitive"); break;
-    default:				text[CLIST_COL_STATE] = g_strdup ("(x) Normal"); break;
-    }
-  text[CLIST_COL_STATE][1] = GTK_WIDGET_VISIBLE (widget) ? 'V' : 'H';
-  for (i = 0; i < CLIST_N_COLS; i++)
-    {
-      gint new_width;
-
-      new_width = gdk_text_width (GTK_WIDGET (clist)->style->font, text[i], strlen (text[i]));
-      if (clist->column[i].width < new_width)
-	gtk_clist_set_column_width (clist, i, new_width);
-    }
 
   row_data = g_mem_chunk_alloc (shell->row_data_mem_chunk);
   row_data->shell = shell;
   row_data->widget = widget;
   row_data->row = gtk_clist_append (clist, text);
   gtk_clist_set_row_data (clist, row_data->row, row_data);
-  row_data->destroy_handler =
-    gtk_signal_connect (GTK_OBJECT (widget),
-			"destroy",
-			GTK_SIGNAL_FUNC (gle_row_data_destroyed),
-			row_data);
-  for (i = 0; i < CLIST_N_COLS; i++)
-    g_free (text[i]);
-
+  if (!GTK_OBJECT_DESTROYED (widget))
+    {
+      row_data->gle_name_handler =
+	gtk_signal_connect (GTK_OBJECT (widget),
+			    "gle-name-changed",
+			    GTK_SIGNAL_FUNC (gle_row_data_gle_name_changed),
+			    row_data);
+      row_data->destroy_handler =
+	gtk_signal_connect (GTK_OBJECT (widget),
+			    "destroy",
+			    GTK_SIGNAL_FUNC (gle_row_data_destroyed),
+			    row_data);
+    }
+  else
+    {
+      row_data->gle_name_handler = 0;
+      row_data->destroy_handler = 0;
+    }
+  g_free (text[CLIST_COL_TYPE]);
+  
   if (GTK_IS_CONTAINER (widget))
     {
       gchar *old_indent;
 
       old_indent = shell->clist_indent;
-      shell->clist_indent = g_strconcat (old_indent, "	 ", NULL);
+      shell->clist_indent = g_strconcat (old_indent, "  ", NULL);
       gtk_container_foreach (GTK_CONTAINER (widget),
 			     (GtkCallback) gle_shell_clist_append_widget,
 			     shell);
@@ -640,7 +746,7 @@ gle_shell_deletion_request (GleShell	*shell)
 
   gtk_widget_destroy (GTK_WIDGET (shell));
 
-  return FALSE;
+  return TRUE;
 }
 
 static void
@@ -669,12 +775,7 @@ gle_shell_row_selected (GtkCList       *clist,
 
 		  row_data = gtk_clist_get_row_data (clist, row);
 		  if (row_data)
-		    {
-		      GtkWidget *dialog;
-
-		      dialog = gle_editor_popup (row_data->widget);
-		      gle_root_connect_object_life (GTK_OBJECT (dialog));
-		    }
+		    gle_popup_widget_editor (row_data->widget);
 		}
 	      else if (event->button == 3)
 		{
@@ -710,7 +811,7 @@ gle_shell_row_selected (GtkCList       *clist,
 			gtk_object_ref (GTK_OBJECT (row_data->widget));
 		      else
 			gtk_object_unref (GTK_OBJECT (row_data->widget));
-		      sprintf (string, "%04d", GTK_OBJECT (row_data->widget)->ref_count);
+		      sprintf (string, "%05d", GTK_OBJECT (row_data->widget)->ref_count);
 		      gtk_clist_set_text (clist, row, CLIST_COL_REFCOUNT, string);
 		    }
 		}
